import random
from ..simple import MungSubstitutor


class MungSubstitutorRandom(MungSubstitutor):
    """
    SubClasses MungSubstituor to give
    us random values from the simple
    munger
    """

    def substitute(self, pwd):
        """
        Random samples 3 items
        from parent class simple
        substitution table
        """

        random_simple_table = random.sample(
            self.substitution_table_simple.items(), 3)
        return self.munger(pwd, random_simple_table)



from gen_password import PasswordGenerator

class DateNameMixin(object):
    """
    Mixin for supporting date and
    name processing.
    """ 
    def name_dob(self, values):
        """
        Generate name/dob passwords.
        This relies on specific values provided in
        the input YAML file for each individual. 
        Namely:

        first_name
        last_name
        dob      

        """
        name_dob_combo = {}

        if values['first_name'] and values['dob']:
            name_dob_combo['first_name_dob'] = self.date_and_name_processor(
                values['first_name'], values['dob'])

        if values['last_name'] and values['dob']:
            name_dob_combo['last_name_dob'] = self.date_and_name_processor(
                values['last_name'], values['dob'])

        name_dob_combo = PasswordGenerator(
            self.cat,
            self.words,
            name_dob_combo,
            self.encryption_dict,
            self.substitutors_dict).process_individual()

        return name_dob_combo

    def date_and_name_processor(self, name, dob):
        """
        Date and Name processor
        """
        name_dob = []
        name_dob.append(name + str(dob))
        name_dob.append(name + str(dob.year))
        name_dob.append(str(dob.year) + name)
        name_dob.append(str(dob.year)[2] + str(dob.year)[3] + name)
        name_dob.append(name + str(dob.year)[2] + str(dob.year)[3])

        return name_dob



import yaml


class ProcessOutputYaml():

    def output_processor(self, ind_dict):
        """
        Output the processed results
        to a YAMl file
        """ 
        output_doc = file('passwords.yaml', 'w')
        yaml.dump(ind_dict, output_doc, default_flow_style=False)



from ..simple import MungSubstitutor


class MungSubstitutorCommon(MungSubstitutor):

    substitution_table_common = {
        'a': '@',
        'b': '8',
        'd': '6',
        'e': '3',
        'g': '9',
        'i': '1',
        'l': '1',
        'o': '0',
        'q': '9',
        's': '$',
        '0': 'O',
        '1': 'I',
        '3': 'E',
        'A': '@',
        'E': '3',
        'I': '1',
        'O': '0',
        'S': '2'
    }

    def substitute(self, pwd):
        return self.munger(pwd, self.substitution_table_common.iteritems())



import bcrypt


class BcryptEncryption():

    def hash(self, pwd):
        """
        Return a bcrypt hash value
        """
        hash_val = bcrypt.hashpw(pwd, bcrypt.gensalt())
        return hash_val



import argparse
from input import ProcessInputYaml, ProcessInputWords
from gen_logo import Logo
from categories import Categories
from output import ProcessOutputYaml


def main():
    """
    Function to kick off the show.
    Accepts a YAML file as input
    """
    logo = Logo()
    logo.generate_logo()
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "yaml",
        help="an input YAML doc of bio and geographical data of the target")
    parser.add_argument(
        "words",
        help="an input text document containing a word list e.g. Linux/Unix words ")
    args = parser.parse_args()
    process_input(args.yaml, args.words)


def process_input(yaml_file, words_file):
    """
    Create a new YAML parsing object
    and dump the content out as a dict
    """
    print "Processing input YAML"
    yaml_to_dict = ProcessInputYaml()
    yaml_to_dict = yaml_to_dict.yaml_processor(yaml_file)
    words_to_list = ProcessInputWords()
    words_to_list = words_to_list.words_processor(words_file)

    for i in yaml_to_dict:
        for x in i['individuals']:
            generate_password_list(x, words_to_list)


def generate_password_list(individual, words_to_list):
    """
    Kick off the password list generation
    """
    individuals_passwords = Categories(
        individual, words_to_list).process_categories()
    dict_to_yaml = ProcessOutputYaml()
    dict_to_yaml.output_processor(individuals_passwords)


if __name__ == "__main__":
    main()



from ..gen_password import PasswordGenerator


class FreeDataGenerator():
    """
    Any data that does not fit into 
    a specific category with advanced
    processing is considered free data.
    This class handles the processing
    of free data.
    """ 
    def process_data(
            self,
            k,
            words,
            free_bio_data,
            encryption_dict,
            substitutors_dict):
        """
        Process the free bio data.
        At the moment we simply pass
        off to the password generator.
        """
        free_bio_passwords = PasswordGenerator(
            k,
            words,
            free_bio_data,
            encryption_dict,
            substitutors_dict).process_individual()
        return free_bio_passwords



import hashlib


class MD5Encryption():
    def hash(self, pwd):
        """
        Return a md5 hash value
        """
        hash_val = hashlib.md5()
        hash_val.update(pwd.encode('utf-8'))
        return hash_val.hexdigest()
